###############################################################################
#
# Copyright (C) 2013 Wind River Systems, Inc.
#
# This code is licensed under the GPLv2.
#
###############################################################################
import sys
# specify the class look-up paths too...
sys.path.append('/usr/share/ovp-ott')
import os
import string
import xml.etree.ElementTree as ET
import ovp_ott_param as OP



###############################################################################
# Class definition
#
# This class implements an XML reader, and some helper methods, which provides
# support for the OVP_OTT tuning sub-system.
###############################################################################
class OVP_OTT_KB:
    fileName = 'ovp_ott_kb.xml' # only one file to be parsed regardless of
                                # how many instances we have here
    fileName = os.path.dirname(os.path.realpath(__file__)) + '/' + fileName

    # Class constructor, initializes the knowledge base file name (if it's not
    # yet initialized
    def __init__(self):
        if (OVP_OTT_KB.fileName == ''):
            OVP_OTT_KB.fileName = os.path.dirname(os.path.realpath(__file__)) +\
                                  '/ovp_ott_kb.xml'


    # Parses the knowledge base file, and returns a list of accepted parameters
    # to the caller. Any other parameter outside of this list will trigger the
    # showHelp method.
    # The returned list is a list of OVP_OTT_PARAM elements, elements which
    # maps on 1:1 basis to the XML ovp_param nodes.
    def parseFile(self):
        paramList = []
        tree = ET.parse(OVP_OTT_KB.fileName)
        root = tree.getroot()

        for e in root.findall('ovp_param'):
            paramElement = OP.OVP_OTT_PARAM()
            paramElement.setName(str(e.find('name').text))
            paramElement.setType(str(e.find('type').text))
            paramElement.setValue(str(e.find('default_value').text))
            paramElement.setDescription(str(e.find('description').text))
            paramElement.setHint(str(e.find('hint').text))
            paramElement.setHelp(str(e.find('help').text))
            paramElement.set_getCommand(str(e.find('get_command').text))
            paramElement.set_setCommand(str(e.find('set_command').text))
            paramElement.set_cmdline_switch(str(e.find('cmdline_switch').text))
            # add the element into the parameter list
            paramList.append(paramElement)

        # sort the list of parameters based on their types (non-tunable first)
        paramList.sort(key=lambda x: x.type, reverse=False)
        return paramList


    # Displays the help screen of the tuner application. It will go through the
    # list of accepted parameters, and will show each parameter command line
    # switch together with a small description of it.
    def showHelp(self):
        pl = self.parseFile()
        print string.center("*", 80, "*")
        print "USAGE:"
        print sys.argv[0] + " COMMAND CMDLINE_OPTIONS [PARAMETERS]"
        print
        print "  Where COMMAND: "
        print "        GET|get|SET|set|HELP|help"
        print

        for e in pl:
            helpString = "  " + e.get_cmdline_switch() + " - " + e.getHelp()
            if e.getType() != 'tunable':
                helpString = "  " + e.get_cmdline_switch() + " -* " + e.getHelp()
            print helpString
        print string.center("*", 80, "*")


    # Return all the command line switches as a single string. Depending on the
    # specified command (get or set), it will add the special ':' character
    # after each command line switch (in the case of set, when a value is also
    # required). This string will be used by the caller, to determine the
    # command line correctness.
    def getOptions(self, getSet):
        pl = self.parseFile()
        optionsStr = ""
        paramReqChar = ""

        if getSet in ("set", "SET"):
            paramReqChar = ":" # parameter is mandatory!

        for e in pl:
            cs = e.get_cmdline_switch()
            optionsStr = optionsStr + cs[1:] + paramReqChar
        return optionsStr


    # Looks up a whole parameter based on it's command line switch. It will
    # return None, if not found in the knowledge base, or otherwise it will
    # return the requested element.
    def findElementByCmdlineSwitch(self, pCmdLineSwitch):
        pl = self.parseFile()
        retVal = None

        for e in pl:
            if e.get_cmdline_switch() == pCmdLineSwitch:
                retVal = e
                break
        return retVal


    # Helper method, displaying some statistic information about the XML
    #knowledge base. It's used only in debugging.
    def showStatistics(self):
        r = self.parseFile()
        print "root element: " + r.tag
        l = r.findall('ovp_param')
        print "number of ovp parameters: " + str(len(l))

