#!/usr/bin/env python
###############################################################################
#
# Copyright (C) 2013 Wind River Systems, Inc.
#
# This code is licensed under the GPLv2.
#
###############################################################################
import sys
# specify the class look-up paths too...
sys.path.append('/usr/share/ovp-ott')
import getopt
import ovp_ott_kb as KB
import ovp_ott_executor as EXEC
import ovp_ott_error_codes as ERR


###############################################################################
### Main logic
###############################################################################

knowledgeBase = KB.OVP_OTT_KB()

# the case when no command line parameters at all are present on the
# command line
if len(sys.argv) == 1:
    knowledgeBase.showHelp()
    sys.exit(ERR.OVP_OTT_ERR_INCOMPLETE_COMMANDLINE)

# The first option right before the tool name shall be get or set
Get_or_Set = sys.argv[1]
if Get_or_Set not in ("GET", "get", "set", "SET", "help", "HELP"):
    knowledgeBase.showHelp()
    sys.exit(ERR.OVP_OTT_ERR_COMMAND_ERROR)

# the optionString is constructed based on set or get desired behavior.
# if set is requested, then each option will have the ':' character
# which means that after such a command line option a MANDATORY parameter
# shall be present (it's somehow, some kind of basic command line
# validation)
try:
    optionString = knowledgeBase.getOptions(Get_or_Set)
    opts, args = getopt.getopt(sys.argv[2:], optionString)
except getopt.GetoptError:
    knowledgeBase.showHelp()
    sys.exit(ERR.OVP_OTT_ERR_CMDLINE_OPTION_ERROR)

# This is the case, when the script is called with no command line
# command, parameter anything.... just the plain script name....
if len(opts) == 0:
    knowledgeBase.showHelp()
    sys.exit(ERR.OVP_OTT_ERR_INCOMPLETE_COMMANDLINE)

# check if we have a set or get operation (the correctness is already
# established above...)
isSetOperation = False
if Get_or_Set in ("set", "SET"):
    isSetOperation = True

# is it Help?
isHelpOperation = False
if Get_or_Set in ("help", "HELP"):
    isHelpOperation = True

# processing the command line options
for opt,arg in opts:
    if opt in ("-h", "--help"):
        knowledgeBase.showHelp()
        sys.exit(ERR.OVP_OTT_ERR_NO_ERROR)
    else:
        # try to find the element in the list of elements
        element = knowledgeBase.findElementByCmdlineSwitch(opt)
        executor = EXEC.OVP_OTT_EXECUTOR()
        if element != None:

            if isHelpOperation:
                shl = "\nHelp: " + element.getHelp()
                shl = shl + "\nHint: " + element.getHint()
                shl = shl + "\nDescription: " + element.getDescription()
                shl = shl + "\nOVP_OTT_IEOM"
                cmd = "cat << 'OVP_OTT_IEOM' " + shl
                executor.executeCmd(cmd)
            else:
                etype = element.getType()

                # Tunable elements - SET operation have a real meaning on this
                if (etype == 'tunable'):
                    if isSetOperation:
                        cmd = element.get_setCommand()
                        executor.executeCmd(cmd, 'OVP_OTT_NEWVAL', arg)
                    elif isHelpOperation:
                        shl = "\nHelp: " + element.getHelp()
                        shl = shl + "\nHint: " + element.getHint()
                        shl = shl + "\nDescription: " + element.getDescription()
                        shl = shl + "\nOVP_OTT_EOM"
                        cmd = "cat << 'OVP_OTT_EOM' " + shl
                        executor.executeCmd(cmd)
                    else:
                        cmd = element.get_getCommand()
                        if cmd != None:
                            executor.executeCmd(cmd)

                # non-tunable operation. doesn't matter if set or get, only a
                # hint will provided to the end-user....
                elif (etype == 'non-tunable'):
                    # in this case doesn't matter if we get the get or set
                    # command, they shall be the same!
                    cmd = element.get_getCommand()
                    paramToShow = element.getHint()
                    executor.executeCmd(cmd, 'OVP_OTT_HINT', paramToShow)

                # unknown element type, report it as error, and exit!
                else:
                    sys.stderr.write("Unknown element type: " + etype)
                    sys.stderr.write("Please consult the documentation!")
                    sys.exit(ERR.OVP_OTT_ERR_UNKNOWN_ELEMENT_TYPE)

sys.exit(ERR.OVP_OTT_ERR_NO_ERROR)

